#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This package contains basic event handlers."""

from boduch.event import subscribe, EventSetPush, EventSetSort, EventHashPush
from boduch.handle.handle import Handle
from boduch.handle.data import HandleHashPush, HandleSetPush, HandleSetSort

subscribe(EventSetPush, HandleSetPush)
subscribe(EventSetSort, HandleSetSort)
subscribe(EventHashPush, HandleHashPush)

__all__=['Handle', 'HandleSetPush', 'HandleSetSort', 'HandleHashPush']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the Set class which is basically a list that
emits events."""

from zope.interface import implements
from boduch.interface import ISet
from boduch.type import Type
from boduch.event import publish, EventSetPush, EventSetPop,\
                         EventSetGet, EventSetSort

class Set(Type):
    """This is the Set class that emits events."""
    implements(ISet)
    def __init__(self, data=[]):
        """Constructor.  Initialize the Set data."""
        self.data=data
        
    def push(self, obj):
        """Push an object on the list.  This operation emits a EventSetPush
        event."""
        publish(EventSetPush, kw={'set':self, 'obj':obj})
        
    def pop(self, index):
        """Pop an object from the list.  This operation emits a EventSetPop
        event."""
        publish(EventSetPop, kw={'set':self, 'index':index})
        obj=self.data.pop(index)
        return obj
        
    def get(self, index):
        """Retrieve an object from the list.  This operation emits a 
        EventSetGet event."""
        publish(EventSetGet, kw={'set':self, 'index':index})
        obj=self.data[index]
        return obj
        
    def sort(self, reverse=False):
        publish(EventSetSort, kw={'set':self, 'reverse':reverse})
        
__all__=['Set']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the basic Event class which serves as the base class
of any subsequent events that may be implemented in this event system."""

import time
from zope.interface import implements
from boduch.interface import IEvent
from boduch.type import Type

class Event(Type):
    """This is the basic Event class which implements the IEvent interface.
    All events in this system should derive from this class."""
    implements(IEvent)
    
    def __init__(self, *args, **kw):
        """This is the Event constructor.  We also initialize the Type class
        here."""
        Type.__init__(self, *args, **kw)
        self.time=time.time()
        
    def get_time(self):
        """Return the time this event occured."""
        return self.time
        
__all__=['Event']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module provides a test case for the Hash class and a test
suite that contains the test case."""

import unittest

from boduch.interface import IHash
from boduch.event import Event, publish, subscribe, unsubscribe
from boduch.handle import Handle
from boduch.data import Hash

class TestHash(unittest.TestCase):
    """This class defines the unit tests for the Hash class."""
    def setUp(self):
        """Instantiate the Hash instance to test with."""
        self.hash_obj=Hash()
        
    def test_A_interface(self):
        """Test the Hash interface."""
        self.assertTrue(IHash.implementedBy(Hash),\
                        'IHash not implemented by Hash.')
        self.assertTrue(IHash.providedBy(self.hash_obj),\
                        'IHash not provided by Hash instance.')
                        
    def test_B_push(self):
        """Test the Hash.push() method."""
        try:
            self.hash_obj.push(('key', 'value'))
        except Exception, e:
            self.fail(str(e))
                  
    def test_C_get(self):
        """Test the Hash.get() method."""
        try:
            self.hash_obj.get('key')
        except Exception, e:
            self.fail(str(e))
            
    def test_D_pop(self):
        """Test the Hash.pop() method."""
        try:
            self.hash_obj.pop('key')
        except Exception, e:
            self.fail(str(e))
            
SuiteHash=unittest.TestLoader().loadTestsFromTestCase(TestHash)
            
__all__=['TestHash', 'SuiteHash']



#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines some simple utility type functionality."""

import types
from boduch.constant import *

def is_type(obj, t):
    """Return true if the specified obj is the specified type."""
    candidates=[]
    if hasattr(obj, '__name__'):
        candidates.append(obj.__name__)
    elif hasattr(obj, '__class__'):
        class_obj=obj.__class__
        candidates.append(class_obj.__name__)
        for base in class_obj.__bases__:
            candidates.append(base.__name__)
    if hasattr(obj, '__bases__'):
        for base in obj.__bases__:
            candidates.append(base.__name__)
    if not candidates and type(t) is types.StringType:
        t=t.upper()
        if t==TYPE_BOOL or t==TYPE_BOOLEAN:
            return type(obj) is types.BooleanType
        elif t==TYPE_INT or t==TYPE_INTEGER:
            return type(obj) is types.IntType
        elif t==TYPE_LONG:
            return type(obj) is types.LongType
        elif t==TYPE_FLOAT:
            return type(obj) is types.FloatType
        elif t==TYPE_STR or t==TYPE_STRING:
            return type(obj) is types.StringType
        elif t==TYPE_UNICODE:
            return type(obj) is types.UnicodeType
        elif t==TYPE_TUPLE:
            return type(obj) is types.TupleType
        elif t==TYPE_LIST:
            return type(obj) is types.ListType
        elif t==TYPE_DICT or t==TYPE_DICTIONARY:
            return type(obj) is types.DictionaryType
    else:
        return t in candidates

__all__=['is_type']



#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the IEventManager interface."""

from zope.interface import Interface, Attribute

class IEventManager(Interface):
    subscriptions=Attribute("""Stores all event subscriptions.""")
    queue=Attribute("""The queue where threaded handles are placed.""")
    
    def get_subscriptions(cls, evt=None):
        """Return the handles for the specified event."""
        
    def publish(cls, evt, args=[], kw={}, atomic=False):
        """Publish the specified event."""
        
    def subscribe(cls, evt, handle):
        """Subscribe the specified handle to the specified event."""
        
    def unsubscribe(cls, evt, handle):
        """Remove the subscription of the specified handle to the specified
        event."""
        
    def prioritize(cls, evt):
        """Sort the list of subscribers to the specified event,
        making the largest handles first."""
        
    def build_handlers(cls, evt, params={}):
        """Build the Handle instances that are currently subcribers to
        the specified event.  The instance is also passed the specified
        keyword parameters."""
    
__all__=['IEventManager']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This package contains the interface modules used in the boduch Python
library."""

from boduch.interface.eventthread import IEventThread
from boduch.interface.event import IEvent
from boduch.interface.eventmanager import IEventManager
from boduch.interface.threadmanager import IThreadManager
from boduch.interface.type import IType
from boduch.interface.handle import IHandle
from boduch.interface.set import ISet
from boduch.interface.hash import IHash

__all__=['IEventThread', 'IEvent', 'IEventManager',\
         'IThreadManager', 'IType', 'IHandle', 'ISet', 'IHash']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the interface for the base type class."""

from zope.interface import Interface, Attribute

class IType(Interface):
    data=Attribute("""The data dictionary associated with the type.""")
    uuid=Attribute("""The uuid of this type.""")
    
    """The base type class defines several operator-overloading methods."""
    def __len__(self):
        """Needed to compute the length of an instance."""
        
    def __eq__(self, other):
        """Needed for instance equivalence comparison."""
        
    def __lt__(self, other):
        """Needed for instance less than comparison."""
        
    def __gt__(self, other):
        """Needed for instance greater than comparison."""
        
    def __cmp__(self, other):
        """Needed for instance sorting comparison."""
        
__all__=['IType']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the core event system functionality."""

from Queue import Queue
from zope.interface import implements
from boduch.interface import IEventManager
from boduch.event import ThreadManager
from boduch.type import is_type

class EventManager(object):
    """This class is the core event system.  This is an abstract class and
    there are therefore no instances.  This class defines two class attributes.
    
    The slock attribute is a Semaphore instance from the builtin threading
    Python library.  This lock exists because each time an event is published,
    a new thread of control is started.
    
    The subscriptions attribute is a dictionary containing all event handlers.
    The key of this dictionary is the event class itself."""
    implements(IEventManager)
    subscriptions={}
    queue=Queue(0)
    
    @classmethod
    def get_subscriptions(cls, evt=None):
        """Return the handle instances that are subscribed to the specified
        event.  If the event is not specified, return all handles."""
        if evt and cls.subscriptions.has_key(evt):
            return cls.subscriptions[evt]
        return cls.subscriptions
    
    @classmethod
    def publish(cls, evt, args=[], kw={}, atomic=False):
        """Publish an event.
        
        The evt parameter is the event class to instantiate.  The args 
        parameter and the kw parameter are passed to the event constructor."""
        if cls.get_subscriptions().has_key(evt):
            params={'event':evt(*args, **kw)}
            handles=cls.build_handlers(evt, params=params)
            if ThreadManager.get_threaded() and not atomic:
                for handle in handles:
                    cls.queue.put(handle)
                    ThreadManager.start_event_thread(cls.queue)
            else:
                for handle in handles:
                    handle.run()
        
    @classmethod
    def subscribe(cls, evt, handle):
        """Subscribe to an event.  The event parameter must inherit from Event
        and the handle parameter must inherit from Handle."""
        if not is_type(evt, "Event"):
            raise TypeError("Invalid event.")
        if not is_type(handle, "Handle"):
            raise TypeError("Invalid handle.")
        if cls.get_subscriptions().has_key(evt):
            if handle not in cls.get_subscriptions(evt=evt):
                cls.subscriptions[evt].append(handle)
                cls.prioritize(evt)
        else:
            cls.subscriptions[evt]=[handle]
            
    @classmethod
    def unsubscribe(cls, evt, handle):
        """Unsubcribe the specified handler from the specified event."""
        if cls.get_subscriptions().has_key(evt):
            cnt=0
            for handle_obj in cls.get_subscriptions(evt=evt):
                if handle==handle_obj:
                    cls.subscriptions[evt].pop(cnt)
                    return
                cnt+=1
                
    @classmethod
    def prioritize(cls, evt):
        """Prioritize the handlers for the specified event in descending
        order."""
        cls.subscriptions[evt].sort(reverse=True)
            
    @classmethod
    def build_handlers(cls, evt, params={}):
        """Build and return a list of event handlers for the specified 
        event."""
        handlers=[]
        for handler in cls.get_subscriptions(evt=evt):
            handlers.append(handler(**params))
        return handlers
    
__all__=['EventManager']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the base event Handler."""

from zope.interface import implements
from boduch.interface import IHandle
from boduch.type import Type

class Handle(Type):
    """The base event handler.  When run, print a simple message.  This
    handler is not very useful on its' own and should be sub-classed."""
    implements(IHandle)
    
    def __init__(self, *args, **kw):
        """Constructor.  Initialize the Type class."""
        Type.__init__(self, *args, **kw)
        
    def run(self):
        """The run method that executes the handle.  This should be redefined
        by sub-classes."""
        raise NotImplementedError("Handle.run()")
    
    def get_event(self):
        """Return the Event instance that caused this handle to be
        instantiated."""
        if self.data.has_key('event'):
            return self.data['event']
        
__all__=['Handle']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from zope.interface import implements
from boduch.interface import IHash
from boduch.type import Type
from boduch.event import publish, EventHashPush, EventHashPop, EventHashGet

class Hash(Type):
    implements(IHash)
    def __init__(self, data={}):
        """Constructor.  Initialize the Hash data."""
        self.data=data
        
    def push(self, pair):
        """Push key-value pair on the dictionary."""
        publish(EventHashPush, kw={'hash':self, 'pair':pair})
        
    def pop(self, key):
        """Pop an object from the dictionary."""
        publish(EventHashPop, kw={'hash':self, 'key':key})
        obj=self.data[key]
        del self.data[key]
        return obj
        
    def get(self, key):
        """Retrieve an object from the dictionary."""
        publish(EventHashGet, kw={'hash':self, 'key':key})
        obj=self.data[key]
        return obj
    
__all__=['Hash']



#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines events related to the data.Hash data type."""

from boduch.event import Event

class EventHashPush(Event):
    """Event that is raised by the Hash.push() method."""
    def __init__(self, *args, **kw):
        Event.__init__(self, *args, **kw)
        
class EventHashPop(Event):
    """Event that is raised by the Hash.pop() method."""
    def __init__(self, *args, **kw):
        Event.__init__(self, *args, **kw)
        
class EventHashGet(Event):
    """Event that is raised by the Hash.get() method."""
    def __init__(self, *args, **kw):
        Event.__init__(self, *args, **kw)
        
__all__=['EventHashPush', 'EventHashPop', 'EventHashGet']
        


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This package provides some basic tools to assist with common tasks
while developing Python applications.

The event package provides a simple implementation of an event system with
a publish-subscribe architecture."""

from boduch.constant import PRIORITY_CRITICAL, PRIORITY_MAJOR, PRIORITY_MINOR,\
                            PRIORITY_TRIVIAL, TYPE_BOOL, TYPE_BOOLEAN,\
                            TYPE_INT, TYPE_INTEGER, TYPE_LONG, TYPE_FLOAT,\
                            TYPE_STR, TYPE_STRING, TYPE_UNICODE, TYPE_TUPLE,\
                            TYPE_LIST, TYPE_DICT, TYPE_DICTIONARY
from boduch import interface
from boduch import event
from boduch import type
from boduch import handle
from boduch import data
from boduch import test

__all__=['PRIORITY_CRITICAL', 'PRIORITY_MAJOR', 'PRIORITY_MINOR',\
         'PRIORITY_TRIVIAL', 'TYPE_BOOL', 'TYPE_BOOLEAN', 'TYPE_INT',\
         'TYPE_INTEGER', 'TYPE_LONG', 'TYPE_FLOAT', 'TYPE_STR', 'TYPE_STRING',\
         'TYPE_UNICODE', 'TYPE_TUPLE', 'TYPE_LIST', 'TYPE_DICT',\
         'TYPE_DICTIONARY', 'interface', 'event', 'type', 'handle', 'data',\
         'test']
__version__='0.0.9'
__author__='Adam Boduch'


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the IHash interface."""

from zope.interface import Interface, Attribute

class IHash(Interface):
    """The basic interface for hash objects."""
    data=Attribute("The hash data.")
    
    def push(self, pair):
        """Push a key-value pair onto the hash."""
        
    def pop(self, key):
        """Remove the object at the specified key from the hash."""
        
    def get(self, key):
        """Return the object at the specified key."""
        
__all__=['IHash']



#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This package contains test cases and test suites for the boduch
Python library."""

from boduch.test.event import TestEvent, SuiteEvent
from boduch.test.set import TestSet, SuiteSet
from boduch.test.hash import TestHash, SuiteHash
from boduch.test.util import run_tests

__all__=['TestEvent', 'TestSet', 'TestHash', 'SuiteEvent', 'SuiteSet',\
         'SuiteHash', 'run_tests']



#!/usr/bin/python

from setuptools import setup, find_packages
setup(name="boduch",\
      version='0.0.9',\
      install_requires = ['zope.interface'],\
      zip_safe=False,\
      packages = find_packages('src'),\
      package_dir = {'':'src'},\
      author = "Adam Boduch",\
      author_email = "adam.boduch@gmail.com",\
      license = "AGPL",\
      url = "http://www.boduch.ca/")

